@@ -52,11 +52,15 @@
 
 #include <QtCore/qdatetime.h>
 #include <QtCore/qurl.h>
+#include <QtCore/qelapsedtimer.h>
 
 #include <QtCore/qdebug.h>
 
 QT_USE_NAMESPACE
 
+int AVFCameraSession::m_defaultCameraIndex;
+QList<AVFCameraInfo> AVFCameraSession::m_cameraDevices;
+
 @interface AVFCameraSessionObserver : NSObject
 {
 @private
@@ -81,6 +85,7 @@ QT_USE_NAMESPACE
     self->m_session = session;
     self->m_captureSession = session->captureSession();
 
+    [m_captureSession retain];
     [[NSNotificationCenter defaultCenter] addObserver:self
                                              selector:@selector(processRuntimeError:)
                                                  name:AVCaptureSessionRuntimeErrorNotification
@@ -99,6 +104,22 @@ QT_USE_NAMESPACE
     return self;
 }
 
+- (void) dealloc
+{
+    [[NSNotificationCenter defaultCenter] removeObserver:self
+                                                    name:AVCaptureSessionRuntimeErrorNotification
+                                                  object:m_captureSession];
+
+    [[NSNotificationCenter defaultCenter] removeObserver:self
+                                                    name:AVCaptureSessionDidStartRunningNotification
+                                                  object:m_captureSession];
+
+    [[NSNotificationCenter defaultCenter] removeObserver:self
+                                                    name:AVCaptureSessionDidStopRunningNotification
+                                                  object:m_captureSession];
+    [m_captureSession release];
+    [super dealloc];
+}
 
 - (void) processRuntimeError:(NSNotification *)notification
 {
@@ -126,7 +147,6 @@ AVFCameraSession::AVFCameraSession(AVFCameraService *service, QObject *parent)
    , m_state(QCamera::UnloadedState)
    , m_active(false)
    , m_videoInput(nil)
-   , m_audioInput(nil)
 {
     m_captureSession = [[AVCaptureSession alloc] init];
     m_observer = [[AVFCameraSessionObserver alloc] initWithCameraSession:this];
@@ -142,20 +162,104 @@ AVFCameraSession::~AVFCameraSession()
         [m_videoInput release];
     }
 
-    if (m_audioInput) {
-        [m_captureSession removeInput:m_audioInput];
-        [m_audioInput release];
-    }
-
     [m_observer release];
     [m_captureSession release];
 }
 
+int AVFCameraSession::defaultCameraIndex()
+{
+    updateCameraDevices();
+    return m_defaultCameraIndex;
+}
+
+const QList<AVFCameraInfo> &AVFCameraSession::availableCameraDevices()
+{
+    updateCameraDevices();
+    return m_cameraDevices;
+}
+
+AVFCameraInfo AVFCameraSession::cameraDeviceInfo(const QByteArray &device)
+{
+    updateCameraDevices();
+
+    Q_FOREACH (const AVFCameraInfo &info, m_cameraDevices) {
+        if (info.deviceId == device)
+            return info;
+    }
+
+    return AVFCameraInfo();
+}
+
+void AVFCameraSession::updateCameraDevices()
+{
+#ifdef Q_OS_IOS
+    // Cameras can't change dynamically on iOS. Update only once.
+    if (!m_cameraDevices.isEmpty())
+        return;
+#else
+    // On OS X, cameras can be added or removed. Update the list every time, but not more than
+    // once every 500 ms
+    static QElapsedTimer timer;
+    if (timer.isValid() && timer.elapsed() < 500) // ms
+        return;
+#endif
+
+    m_defaultCameraIndex = -1;
+    m_cameraDevices.clear();
+
+    AVCaptureDevice *defaultDevice = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
+    NSArray *videoDevices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
+    for (AVCaptureDevice *device in videoDevices) {
+        if (defaultDevice && [defaultDevice.uniqueID isEqualToString:device.uniqueID])
+            m_defaultCameraIndex = m_cameraDevices.count();
+
+        AVFCameraInfo info;
+        info.deviceId = QByteArray([[device uniqueID] UTF8String]);
+        info.description = QString::fromNSString([device localizedName]);
+
+        // There is no API to get the camera sensor orientation, however, cameras are always
+        // mounted in landscape on iDevices.
+        //   - Back-facing cameras have the top side of the sensor aligned with the right side of
+        //     the screen when held in portrait ==> 270 degrees clockwise angle
+        //   - Front-facing cameras have the top side of the sensor aligned with the left side of
+        //     the screen when held in portrait ==> 270 degrees clockwise angle
+        // On Mac OS, the position will always be unspecified and the sensor orientation unknown.
+        switch (device.position) {
+        case AVCaptureDevicePositionBack:
+            info.position = QCamera::BackFace;
+            info.orientation = 270;
+            break;
+        case AVCaptureDevicePositionFront:
+            info.position = QCamera::FrontFace;
+            info.orientation = 270;
+            break;
+        default:
+            info.position = QCamera::UnspecifiedPosition;
+            info.orientation = 0;
+            break;
+        }
+
+        m_cameraDevices.append(info);
+    }
+
+#ifndef Q_OS_IOS
+    timer.restart();
+#endif
+}
+
 void AVFCameraSession::setVideoOutput(AVFVideoRendererControl *output)
 {
     m_videoOutput = output;
     if (output)
-        output->configureAVCaptureSession(m_captureSession);
+        output->configureAVCaptureSession(this);
+}
+
+AVCaptureDevice *AVFCameraSession::videoCaptureDevice() const
+{
+    if (m_videoInput)
+        return m_videoInput.device;
+
+    return 0;
 }
 
 QCamera::State AVFCameraSession::state() const
@@ -176,10 +280,9 @@ void AVFCameraSession::setState(QCamera::State newState)
     QCamera::State oldState = m_state;
     m_state = newState;
 
-    //attach audio and video inputs during Unloaded->Loaded transition
-    if (oldState == QCamera::UnloadedState) {
-        attachInputDevices();
-    }
+    //attach video input during Unloaded->Loaded transition
+    if (oldState == QCamera::UnloadedState)
+        attachVideoInputDevice();
 
     if (m_state == QCamera::ActiveState) {
         Q_EMIT readyToConfigureConnections();
@@ -221,7 +324,7 @@ void AVFCameraSession::processSessionStopped()
     }
 }
 
-void AVFCameraSession::attachInputDevices()
+void AVFCameraSession::attachVideoInputDevice()
 {
     //Attach video input device:
     if (m_service->videoDeviceControl()->isDirty()) {
@@ -249,29 +352,6 @@ void AVFCameraSession::attachInputDevices()
             }
         }
     }
-
-    //Attach audio input device:
-    if (m_service->audioInputSelectorControl()->isDirty()) {
-        if (m_audioInput) {
-            [m_captureSession removeInput:m_audioInput];
-            [m_audioInput release];
-            m_audioInput = 0;
-        }
-
-        AVCaptureDevice *audioDevice = m_service->audioInputSelectorControl()->createCaptureDevice();
-
-        NSError *error = nil;
-        m_audioInput = [AVCaptureDeviceInput
-                deviceInputWithDevice:audioDevice
-                error:&error];
-
-        if (!m_audioInput) {
-            qWarning() << "Failed to create audio device input";
-        } else {
-            [m_audioInput retain];
-            [m_captureSession addInput:m_audioInput];
-        }
-    }
 }
 
 #include "moc_avfcamerasession.cpp"
