@@ -44,6 +44,7 @@
 #include "avfcamerasession.h"
 #include "avfcameraservice.h"
 #include "avfcameracontrol.h"
+#include "avfaudioinputselectorcontrol.h"
 
 #include <QtCore/qurl.h>
 #include <QtCore/qfileinfo.h>
@@ -122,6 +123,7 @@ QT_USE_NAMESPACE
 AVFMediaRecorderControl::AVFMediaRecorderControl(AVFCameraService *service, QObject *parent)
    : QMediaRecorderControl(parent)
    , m_cameraControl(service->cameraControl())
+   , m_audioInputControl(service->audioInputSelectorControl())
    , m_session(service->session())
    , m_connected(false)
    , m_state(QMediaRecorder::StoppedState)
@@ -130,21 +132,29 @@ AVFMediaRecorderControl::AVFMediaRecorderControl(AVFCameraService *service, QObj
    , m_recordingFinished(false)
    , m_muted(false)
    , m_volume(1.0)
+   , m_audioInput(nil)
 {
     m_movieOutput = [[AVCaptureMovieFileOutput alloc] init];
     m_recorderDelagate = [[AVFMediaRecorderDelegate alloc] initWithRecorder:this];
 
     connect(m_cameraControl, SIGNAL(stateChanged(QCamera::State)), SLOT(updateStatus()));
     connect(m_cameraControl, SIGNAL(statusChanged(QCamera::Status)), SLOT(updateStatus()));
-    connect(m_cameraControl, SIGNAL(captureModeChanged(QCamera::CaptureModes)), SLOT(reconnectMovieOutput()));
-
-    reconnectMovieOutput();
+    connect(m_cameraControl, SIGNAL(captureModeChanged(QCamera::CaptureModes)), SLOT(setupSessionForCapture()));
+    connect(m_session, SIGNAL(readyToConfigureConnections()), SLOT(setupSessionForCapture()));
+    connect(m_session, SIGNAL(stateChanged(QCamera::State)), SLOT(setupSessionForCapture()));
 }
 
 AVFMediaRecorderControl::~AVFMediaRecorderControl()
 {
-    if (m_movieOutput)
+    if (m_movieOutput) {
         [m_session->captureSession() removeOutput:m_movieOutput];
+        [m_movieOutput release];
+    }
+
+    if (m_audioInput) {
+        [m_session->captureSession() removeInput:m_audioInput];
+        [m_audioInput release];
+    }
 
     [m_recorderDelagate release];
 }
@@ -315,13 +325,39 @@ void AVFMediaRecorderControl::handleRecordingFailed(const QString &message)
     Q_EMIT error(QMediaRecorder::ResourceError, message);
 }
 
-void AVFMediaRecorderControl::reconnectMovieOutput()
+void AVFMediaRecorderControl::setupSessionForCapture()
 {
     //adding movie output causes high CPU usage even when while recording is not active,
-    //connect it only while video capture mode is enabled
+    //connect it only while video capture mode is enabled.
+    // Similarly, connect the Audio input only in that mode, since it's only necessary
+    // when recording anyway. Adding an Audio input will trigger the microphone permission
+    // request on iOS, but it shoudn't do so until we actually try to record.
     AVCaptureSession *captureSession = m_session->captureSession();
 
-    if (!m_connected && m_cameraControl->captureMode().testFlag(QCamera::CaptureVideo)) {
+    if (!m_connected
+            && m_cameraControl->captureMode().testFlag(QCamera::CaptureVideo)
+            && m_session->state() != QCamera::UnloadedState) {
+
+        // Add audio input
+        // Allow recording even if something wrong happens with the audio input initialization
+        AVCaptureDevice *audioDevice = m_audioInputControl->createCaptureDevice();
+        if (!audioDevice) {
+            qWarning("No audio input device available");
+        } else {
+            NSError *error = nil;
+            m_audioInput = [AVCaptureDeviceInput deviceInputWithDevice:audioDevice error:&error];
+
+            if (!m_audioInput) {
+                qWarning() << "Failed to create audio device input";
+            } else if (![captureSession canAddInput:m_audioInput]) {
+                qWarning() << "Could not connect the audio input";
+                m_audioInput = 0;
+            } else {
+                [m_audioInput retain];
+                [captureSession addInput:m_audioInput];
+            }
+        }
+
         if ([captureSession canAddOutput:m_movieOutput]) {
             [captureSession addOutput:m_movieOutput];
             m_connected = true;
@@ -329,8 +365,18 @@ void AVFMediaRecorderControl::reconnectMovieOutput()
             Q_EMIT error(QMediaRecorder::ResourceError, tr("Could not connect the video recorder"));
             qWarning() << "Could not connect the video recorder";
         }
-    } else if (m_connected && !m_cameraControl->captureMode().testFlag(QCamera::CaptureVideo)) {
+    } else if (m_connected
+               && (!m_cameraControl->captureMode().testFlag(QCamera::CaptureVideo)
+                   || m_session->state() != QCamera::ActiveState)) {
+
         [captureSession removeOutput:m_movieOutput];
+
+        if (m_audioInput) {
+            [captureSession removeInput:m_audioInput];
+            [m_audioInput release];
+            m_audioInput = nil;
+        }
+
         m_connected = false;
     }
 
