@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -48,6 +40,7 @@
 #include "camerabinimageencoder.h"
 #include "camerabincontrol.h"
 #include "camerabinmetadata.h"
+#include "camerabininfocontrol.h"
 
 #ifdef HAVE_GST_PHOTOGRAPHY
 #include "camerabinexposure.h"
@@ -89,8 +82,9 @@
 
 QT_BEGIN_NAMESPACE
 
-CameraBinService::CameraBinService(const QString &service, QObject *parent):
-    QMediaService(parent)
+CameraBinService::CameraBinService(GstElementFactory *sourceFactory, QObject *parent):
+    QMediaService(parent),
+    m_cameraInfoControl(0)
 {
     m_captureSession = 0;
     m_metaDataControl = 0;
@@ -106,39 +100,45 @@ CameraBinService::CameraBinService(const QString &service, QObject *parent):
 #endif
     m_imageCaptureControl = 0;
 
-    if (service == Q_MEDIASERVICE_CAMERA) {
-        m_captureSession = new CameraBinSession(this);
-        m_videoInputDevice = new QGstreamerVideoInputDeviceControl(
-                    m_captureSession->buildCameraSource(), m_captureSession);
-        m_imageCaptureControl = new CameraBinImageCapture(m_captureSession);
+    m_captureSession = new CameraBinSession(sourceFactory, this);
+    m_videoInputDevice = new QGstreamerVideoInputDeviceControl(sourceFactory, m_captureSession);
+    m_imageCaptureControl = new CameraBinImageCapture(m_captureSession);
 
-        connect(m_videoInputDevice, SIGNAL(selectedDeviceChanged(QString)),
-                m_captureSession, SLOT(setDevice(QString)));
+    connect(m_videoInputDevice, SIGNAL(selectedDeviceChanged(QString)),
+            m_captureSession, SLOT(setDevice(QString)));
 
-        if (m_videoInputDevice->deviceCount())
-            m_captureSession->setDevice(m_videoInputDevice->deviceName(m_videoInputDevice->selectedDevice()));
+    if (m_videoInputDevice->deviceCount())
+        m_captureSession->setDevice(m_videoInputDevice->deviceName(m_videoInputDevice->selectedDevice()));
 
 #if defined(Q_WS_MAEMO_6) && defined(__arm__) && defined(HAVE_WIDGETS)
-        m_videoRenderer = new QGstreamerGLTextureRenderer(this);
+    m_videoRenderer = new QGstreamerGLTextureRenderer(this);
 #else
-        m_videoRenderer = new QGstreamerVideoRenderer(this);
+    m_videoRenderer = new QGstreamerVideoRenderer(this);
 #endif
 
 #ifdef Q_WS_MAEMO_6
-        m_videoWindow = new QGstreamerVideoWindow(this, "omapxvsink");
+    m_videoWindow = new QGstreamerVideoWindow(this, "omapxvsink");
 #else
-        m_videoWindow = new QGstreamerVideoWindow(this);
+    m_videoWindow = new QGstreamerVideoWindow(this);
 #endif
+    // If the GStreamer sink element is not available (xvimagesink), don't provide
+    // the video window control since it won't work anyway.
+    if (!m_videoWindow->videoSink()) {
+        delete m_videoWindow;
+        m_videoWindow = 0;
+    }
 
 #if defined(HAVE_WIDGETS)
-        m_videoWidgetControl = new QGstreamerVideoWidgetControl(this);
-#endif
+    m_videoWidgetControl = new QGstreamerVideoWidgetControl(this);
 
+    // If the GStreamer sink element is not available (xvimagesink or ximagesink), don't provide
+    // the video widget control since it won't work anyway.
+    // QVideoWidget will fall back to QVideoRendererControl in that case.
+    if (!m_videoWidgetControl->videoSink()) {
+        delete m_videoWidgetControl;
+        m_videoWidgetControl = 0;
     }
-    if (!m_captureSession) {
-        qWarning() << Q_FUNC_INFO << "Service type is not supported:" << service;
-        return;
-    }
+#endif
 
     m_audioInputSelector = new QGstreamerAudioInputSelector(this);
     connect(m_audioInputSelector, SIGNAL(activeInputChanged(QString)), m_captureSession, SLOT(setCaptureDevice(QString)));
@@ -244,6 +244,12 @@ QMediaControl *CameraBinService::requestControl(const char *name)
     if (qstrcmp(name, QCameraViewfinderSettingsControl_iid) == 0)
         return m_captureSession->viewfinderSettingsControl();
 
+    if (qstrcmp(name, QCameraInfoControl_iid) == 0) {
+        if (!m_cameraInfoControl)
+            m_cameraInfoControl = new CameraBinInfoControl(m_captureSession->sourceFactory(), this);
+        return m_cameraInfoControl;
+    }
+
     return 0;
 }
 
