@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -201,17 +193,6 @@ void MFAudioDecoderControl::handleMediaSourceReady()
     if (mediaType) {
         m_sourceOutputFormat = m_audioFormat;
         QAudioFormat af = m_audioFormat;
-        GUID subType;
-        if (SUCCEEDED(mediaType->GetGUID(MF_MT_SUBTYPE, &subType))) {
-            if (subType == MFAudioFormat_Float) {
-                m_sourceOutputFormat.setSampleType(QAudioFormat::Float);
-            } else {
-                m_sourceOutputFormat.setSampleType(QAudioFormat::SignedInt);
-            }
-        }
-        if (m_sourceOutputFormat.sampleType() != QAudioFormat::Float) {
-            m_sourceOutputFormat.setByteOrder(QAudioFormat::LittleEndian);
-        }
 
         UINT32 val = 0;
         if (SUCCEEDED(mediaType->GetUINT32(MF_MT_AUDIO_NUM_CHANNELS, &val))) {
@@ -224,6 +205,20 @@ void MFAudioDecoderControl::handleMediaSourceReady()
             m_sourceOutputFormat.setSampleSize(int(val));
         }
 
+        GUID subType;
+        if (SUCCEEDED(mediaType->GetGUID(MF_MT_SUBTYPE, &subType))) {
+            if (subType == MFAudioFormat_Float) {
+                m_sourceOutputFormat.setSampleType(QAudioFormat::Float);
+            } else if (m_sourceOutputFormat.sampleSize() == 8) {
+                m_sourceOutputFormat.setSampleType(QAudioFormat::UnSignedInt);
+            } else {
+                m_sourceOutputFormat.setSampleType(QAudioFormat::SignedInt);
+            }
+        }
+        if (m_sourceOutputFormat.sampleType() != QAudioFormat::Float) {
+            m_sourceOutputFormat.setByteOrder(QAudioFormat::LittleEndian);
+        }
+
         if (m_audioFormat.sampleType() != QAudioFormat::Float
             && m_audioFormat.sampleType() != QAudioFormat::SignedInt) {
             af.setSampleType(m_sourceOutputFormat.sampleType());
@@ -244,7 +239,6 @@ void MFAudioDecoderControl::handleMediaSourceReady()
     }
 
     if (m_sourceResolver->mediaSource()) {
-        IMFPresentationDescriptor *pd = 0;
         if (mediaType && m_resampler) {
             HRESULT hr = S_OK;
             hr = m_resampler->SetInputType(m_mfInputStreamID, mediaType, 0);
@@ -254,9 +248,11 @@ void MFAudioDecoderControl::handleMediaSourceReady()
                 qWarning() << "MFAudioDecoderControl: failed to SetInputType of resampler" << hr;
             }
         }
+        IMFPresentationDescriptor *pd;
         if (SUCCEEDED(m_sourceResolver->mediaSource()->CreatePresentationDescriptor(&pd))) {
             UINT64 duration = 0;
             pd->GetUINT64(MF_PD_DURATION, &duration);
+            pd->Release();
             duration /= 10000;
             if (m_duration != qint64(duration)) {
                 m_duration = qint64(duration);
